# -*- coding: utf-8 -*-
# noinspection PyUnresolvedReferences
from loguru import logger

# noinspection PyUnresolvedReferences
from diff_1c.__about__ import __version__
# noinspection PyUnresolvedReferences
from diff_1c.main import Processor



# -*- coding: utf-8 -*-
import os
import shutil
import subprocess
import tempfile
from pathlib import Path

import sys
from loguru import logger

from cjk_commons.settings import SettingsError, get_attribute, get_path_attribute, get_settings
from diff_1c.__about__ import APP_AUTHOR, APP_NAME
from parse_1c_build import Parser

logger.disable(__name__)


class Processor:
    def __init__(self, **kwargs):
        settings_file_path = get_path_attribute(
            kwargs, 'settings_file_path', default_path=Path('settings.yaml'), is_dir=False, check_if_exists=False)
        self.settings = get_settings(settings_file_path, app_name=APP_NAME, app_author=APP_AUTHOR)

        self.tool = get_attribute(kwargs, 'tool', self.settings, 'default_tool', 'kdiff3').lower()
        if self.tool not in self.settings['tools']:
            raise SettingsError('Tool Incorrect')

        self.tool_path = Path(self.settings['tools'][self.tool])
        if not self.tool_path.is_file():
            raise SettingsError('Tool Not Exists')

        self.exclude_file_names = get_attribute(kwargs, 'exclude_file_names', self.settings, 'exclude_file_names', [])
        self.name_format = get_attribute(kwargs, 'name_format', self.settings, 'name_format', 'tortoisegit').lower()

    def run(self, base_file_fullpath: Path, mine_file_fullpath: Path, bname: str = '', yname: str = '') -> None:
        # base
        base_is_excluded = False
        if bname:
            if self.name_format == 'tortoisegit':
                bname_file_fullpath = Path(bname.split(':')[0])
            else:
                bname_file_fullpath = Path(bname.split(':')[0])
        else:
            bname_file_fullpath = Path.cwd()
        base_source_dir_fullpath = None
        if bname_file_fullpath.name not in self.exclude_file_names:
            base_file_fullpath_suffix = base_file_fullpath.suffix
            base_temp_file, base_temp_file_fullname = tempfile.mkstemp(base_file_fullpath_suffix)
            os.close(base_temp_file)
            shutil.copyfile(str(base_file_fullpath), base_temp_file_fullname)
            base_source_dir_fullpath = Path(tempfile.mkdtemp())
            Parser().run(Path(base_temp_file_fullname), base_source_dir_fullpath)
            Path(base_temp_file_fullname).unlink()
        else:
            base_is_excluded = True

        # mine
        mine_is_excluded = False
        if yname:
            if self.name_format == 'tortoisegit':
                yname_file_fullpath = Path(yname.split(':')[0])
            else:
                yname_file_fullpath = Path(yname.split(':')[0])
        else:
            yname_file_fullpath = Path.cwd()
        mine_source_dir_fullpath = None
        if yname_file_fullpath.name not in self.exclude_file_names:
            mine_file_fullpath_suffix = mine_file_fullpath.suffix
            mine_temp_file, mine_temp_file_fullname = tempfile.mkstemp(mine_file_fullpath_suffix)
            os.close(mine_temp_file)
            shutil.copyfile(str(mine_file_fullpath), mine_temp_file_fullname)
            mine_source_dir_fullpath = Path(tempfile.mkdtemp())
            Parser().run(Path(mine_temp_file_fullname), mine_source_dir_fullpath)
            Path(mine_temp_file_fullname).unlink()
        else:
            mine_is_excluded = True

        tool_args = [str(self.tool_path)]
        if self.tool == 'kdiff3':
            # base
            if base_is_excluded:
                tool_args += ['--cs', 'EncodingForA=UTF-8', str(base_file_fullpath)]
            else:
                tool_args += ['--cs', 'EncodingForA=windows-1251', str(base_source_dir_fullpath)]
            if bname is not None:
                tool_args += ['--L1', bname]

            # mine
            if mine_is_excluded:
                tool_args += ['--cs', 'EncodingForB=UTF-8', str(mine_file_fullpath)]
            else:
                tool_args += ['--cs', 'EncodingForB=windows-1251', str(mine_source_dir_fullpath)]
            if yname is not None:
                tool_args += ['--L2', yname]
        exit_code = subprocess.check_call(tool_args)
        if not exit_code == 0:
            raise Exception(f'Diff files \'{base_file_fullpath}\' and \'{mine_file_fullpath}\' failed')


def run(args) -> None:
    logger.enable('cjk_commons')
    logger.enable('parse_1c_build')
    logger.enable(__name__)
    try:
        processor = Processor(name_format=args.name_format, tool=args.tool)

        base_file_fullpath = Path(args.base)
        mine_file_fullpath = Path(args.mine)
        bname = args.bname
        yname = args.yname

        processor.run(base_file_fullpath, mine_file_fullpath, bname, yname)
    except Exception as e:
        logger.exception(e)
        sys.exit(1)



# -*- coding: utf-8 -*-
import sys

from cjk_commons.logging_ import add_loggers
from diff_1c import logger
from diff_1c.cli import get_argparser


def run() -> None:
    argparser = get_argparser()
    args = argparser.parse_args(sys.argv[1:])
    add_loggers(args, logger)
    args.func(args)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['diff_1c']

package_data = \
{'': ['*']}

install_requires = \
['cjk-commons', 'loguru', 'parse-1c-build']

entry_points = \
{'console_scripts': ['diff1c = diff_1c.__main__:run']}

setup_kwargs = {
    'name': 'diff-1c',
    'version': '6.5.5',
    'description': 'Diff utility for 1C:Enterprise',
    'long_description': 'Утилита для сравнения *epf*-, *erf*-, *ert*- и *md*-файлов\n===\n\nЧто делает\n---\n\nФайлы разбираются с помощью пакета [parse-1c-build][1] в каталоги, которые затем сравниваются указанной в аргументах \nкомандной строки утилитой сравнения. Поддерживаются AraxisMerge, ExamDiff, KDiff3, WinMerge.\n\nПри установке пакета в каталоге скриптов интерпретатора Python создаётся исполняемый файл *diff1c.exe*.\n\nТребования\n---\n\n- Windows\n- Python 3.7 и выше. Каталоги интерпретатора и скриптов Python должны быть прописаны в переменной окружения Path\n- Пакет [parse-1c-build][1] с необходимыми настройками\n\n[1]: https://github.com/Cujoko/parse-1c-build\n',
    'author': 'Cujoko',
    'author_email': 'cujoko@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Cujoko/diff-1c',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
from argparse import ArgumentParser

from cjk_commons.logging_ import add_logging_arguments
from diff_1c import __version__
from diff_1c.main import run


def get_argparser() -> ArgumentParser:
    parser = ArgumentParser(
        prog='diff1c', description='Diff utility for 1C:Enterprise files', add_help=False)
    parser.set_defaults(func=run)
    parser.add_argument(
        '-h', '--help',
        action='help',
        help='Show this help message and exit'
    )
    parser.add_argument(
        '-v', '--version',
        action='version',
        version=f'%(prog)s, ver. {__version__}',
        help='Show version'
    )
    parser.add_argument(
        '--bname',
        help='the window title for the base file'
    )
    parser.add_argument(
        '--yname',
        help='the window title for your file'
    )
    parser.add_argument(
        '--name-format',
        choices=['TortoiseGit'],
        default='TortoiseGit',
        help='name format'
    )
    parser.add_argument(
        '--tool',
        choices=['KDiff3', 'AraxisMerge', 'WinMerge', 'ExamDiff'],
        default='KDiff3',
        help='external diff program'
    )
    parser.add_argument(
        'base',
        help='the original file without your changes'
    )
    parser.add_argument(
        'mine',
        help='your own file, with your changes'
    )
    add_logging_arguments(parser)
    return parser
